@@ -35,6 +35,7 @@ class TreesappTester(unittest.TestCase):
     def test_abundance(self):
         from treesapp.abundance import abundance
         from treesapp.file_parsers import read_classification_table
+        from treesapp.phylo_seq import assignments_to_pqueries
         from .testing_utils import get_test_data
         classification_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
         # Copy the classification table to replace after overwrite
@@ -45,43 +46,59 @@ class TreesappTester(unittest.TestCase):
                                   "--reverse", get_test_data("test_TarA.2.fq"),
                                   "--pairing", "pe",
                                   "--num_procs", str(self.num_procs),
-                                  "--delete"]
+                                  "--overwrite"]
         abund_dict = abundance(abundance_command_list)["test_TarA"]
         post_lines = read_classification_table(get_test_data(classification_table))
-        # Ensure no lines were removed
+        # Ensure no lines were added or removed
         self.assertEqual(len(pre_lines), len(post_lines))
         # Ensure the name of the sample is substituted for the sample ID
         self.assertEqual({"test_TarA"}, set([line[0] for line in post_lines]))
+        # Ensure the value
         self.assertEqual(148, round(sum(abund_dict.values())))
+
+        # Test an interleaved fastq file with TPM relative abundance measure
+        abundance_command_list = ["--treesapp_output", self.ts_assign_output,
+                                  "--reads", get_test_data("test_TarA.fastq.gz"),
+                                  "--pairing", "pe",
+                                  "--metric", "tpm",
+                                  "--num_procs", str(self.num_procs),
+                                  "--delete", "--overwrite"]
+        abundance(abundance_command_list)
+
+        pqueries = assignments_to_pqueries(read_classification_table(get_test_data(classification_table)))
+        abundant_pqueries = []
+        for rp in pqueries:
+            abundant_pqueries += [pq for pq in pqueries[rp] if pq.abundance > 0.0]
+        self.assertEqual(33702, round(sum([pq.abundance for pq in abundant_pqueries])))
+
         # Replace the classification table
         copyfile(os.path.join(self.ts_assign_output, "tmp.tsv"), classification_table)
         os.remove(os.path.join(self.ts_assign_output, "tmp.tsv"))
         return
 
-    def test_assign_prot(self):
+    def test_assign(self):
         ref_pkgs = ["McrA", "M0702", "S0001"]
-        from treesapp.assign import assign
+        from treesapp import assign
+        from .testing_utils import get_test_data
         from treesapp.file_parsers import read_classification_table
+        from treesapp.phylo_seq import assignments_to_pqueries
         assign_commands_list = ["--fastx_input", self.aa_test_fa,
                                 "--targets", ','.join(ref_pkgs),
                                 "--refpkg_dir", self.refpkg_dir,
                                 "--num_procs", str(self.num_procs),
                                 "-m", "prot",
+                                "--hmm_coverage", str(20),
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "relaxed",
                                 "--placement_summary", "max_lwr",
                                 "--trim_align", "--overwrite", "--delete", "--svm"]
-        assign(assign_commands_list)
+        assign.assign(assign_commands_list)
         lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
         self.assertEqual(15, len(lines))
         self.assertTrue(os.path.isfile("./TreeSAPP_assign/final_outputs/marker_test_suite_classified.faa"))
-        return
 
-    def test_assign_dna(self):
+        # Test nucleotide sequence input
         ref_pkgs = ["M0701", "M0702"]
-        from treesapp import assign
-        from .testing_utils import get_test_data
-        from treesapp.file_parsers import read_classification_table
         assign_commands_list = ["--fastx_input", self.nt_test_fa,
                                 "--targets", ','.join(ref_pkgs),
                                 "--num_procs", str(self.num_procs),
@@ -95,9 +112,13 @@ class TreesappTester(unittest.TestCase):
                                  "--reads", get_test_data("SRR3669912_1.fastq"),
                                  "--reverse", get_test_data("SRR3669912_2.fastq")]
         assign.assign(assign_commands_list)
-        lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
+        assignments_tbl = "./TreeSAPP_assign/final_outputs/marker_contig_map.tsv"
+        lines = read_classification_table(assignments_tbl)
         self.assertEqual(6, len(lines))
-        classified_seqs = [line[1].split()[0] for line in lines]
+        classified_seqs = set()
+        pqueries = assignments_to_pqueries(lines)
+        for rp_pqs in pqueries.values():
+            classified_seqs.update({pq.seq_name for pq in rp_pqs if pq.abundance >= 0.0})
         self.assertTrue("LYOS01000003.1:168824-170509_1" in classified_seqs)
         self.assertTrue(os.path.isfile("./TreeSAPP_assign/final_outputs/marker_test_suite_classified.fna"))
         return
@@ -139,7 +160,8 @@ class TreesappTester(unittest.TestCase):
                                 "--output", "./TreeSAPP_create",
                                 "--num_procs", str(self.num_procs),
                                 "--max_examples", str(100),
-                                "--trim_align", "--cluster", "--fast", "--headless", "--overwrite", "--delete"]
+                                "--trim_align", "--cluster", "--fast", "--headless",
+                                "--overwrite", "--delete", "--deduplicate"]
         create(create_commands_list)
         test_refpkg = ReferencePackage()
         test_refpkg.f__json = "./TreeSAPP_create/final_outputs/Crt_build.pkl"
@@ -309,11 +331,12 @@ class TreesappTester(unittest.TestCase):
         self.assertEqual(max_ex, len(rank_list))
         return
 
-    def test_update_resolve(self):
+    def test_update(self):
         from treesapp.commands import update
         from treesapp.refpkg import ReferencePackage
         from .testing_utils import get_test_data
-        update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
+        # Test with re-training and the resolve workflow
+        update_command_list = ["--fastx_input", get_test_data("ENOG4111FIN_PF03967_seed.faa"),
                                "--refpkg_path", self.puha_pkl,
                                "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
                                "--output", "./TreeSAPP_update",
@@ -327,14 +350,9 @@ class TreesappTester(unittest.TestCase):
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(49, test_refpkg.num_seqs)
-        return
 
-    def test_update_seqs2lineage(self):
-        from treesapp.commands import update
-        from treesapp.refpkg import ReferencePackage
-        from .testing_utils import get_test_data
-        update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
-                               "--refpkg_path", self.puha_pkl,
+        # Test the workflow when seqs2lineage table is provided and no training
+        update_command_list = ["--refpkg_path", self.puha_pkl,
                                "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
                                "--seqs2lineage", get_test_data("SwissProt_PuhA_seqs2lineage.txt"),
                                "--output", "./TreeSAPP_update",
@@ -349,6 +367,8 @@ class TreesappTester(unittest.TestCase):
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(49, test_refpkg.num_seqs)
+        self.assertEqual((-2.7691, 7.0), test_refpkg.pfit)
+        self.assertTrue(test_refpkg.svc is not None)
         return
 
     # def test_tmp(self):
