@@ -6,13 +6,16 @@ import os
 import logging
 import sys
 import re
+import shutil
 
-from treesapp.fasta import write_new_fasta, FASTA
 from treesapp.external_command_interface import launch_write_command
-from treesapp.classy import Evaluator
-from treesapp.refpkg import ReferencePackage
+from treesapp import refpkg
 from treesapp.entrez_utils import EntrezRecord
+from treesapp import fasta
+from treesapp import classy
 from treesapp import taxonomic_hierarchy
+from treesapp import assign
+from treesapp import file_parsers
 
 
 def parse_distances(classification_lines):
@@ -28,7 +31,7 @@ def parse_distances(classification_lines):
     return distances
 
 
-def determine_containment(marker_eval_inst: Evaluator):
+def determine_containment(marker_eval_inst: classy.Evaluator):
     """
     Determines the accuracy of sequence classifications of all sequences contained at different taxonomic ranks
 
@@ -161,6 +164,20 @@ def get_unclassified_rank(pos, split_lineage):
     return pos
 
 
+def check_lineage_compatibility(lineage_map: dict, taxonomy: taxonomic_hierarchy.TaxonomicHierarchy) -> None:
+    """Ensures that all the lineages are rooted."""
+    unrooted = False
+    for num_id, lineage in lineage_map.items():
+        # Ensure the query lineages are rooted and emit a warning if this is not the case
+        query_split = lineage.split(taxonomy.lin_sep)
+        if query_split[0] != taxonomy.root_taxon:
+            unrooted = True
+            lineage_map[num_id] = taxonomy.root_taxon + taxonomy.lin_sep + lineage
+    if unrooted:
+        logging.warning("Unrooted lineages have been rooted.\n")
+    return
+
+
 def get_testable_lineages_for_rank(ref_lineage_map: dict, query_lineage_map: dict, rank: str,
                                    taxonomy: taxonomic_hierarchy.TaxonomicHierarchy) -> list:
     """
@@ -259,80 +276,39 @@ def pick_taxonomic_representatives(ref_seqs: dict, taxonomic_filter_stats: dict,
     return dereplicated_lineages, taxonomic_filter_stats
 
 
-def str_list_index(lst: list, query: str):
-    i = 0
-    while i < len(lst):
-        if query == lst[i]:
-            break
-        i += 1
-    return i
-
-
-def same_lineage(target_lineage: str, candidate_lineage: str) -> bool:
-    """
-    Identify which candidate lineages belong to target lineage and are at least as specific
-    1. "Root; Bacteria", "Root"                           False
-    2. "Root; Bacteria", "Root; Archaea"                  False
-    3. "Root; Bacteria", "Root; Bacteria"                 True
-    4. "Root; Bacteria", "Root; Bacteria; Proteobacteria" True
-    5. "Bacteria", "Root; Bacteria; Proteobacteria"       True
-    Runs in O(n) complexity
-
-    :param target_lineage:
-    :param candidate_lineage:
-    :return: Boolean
-    """
-    target_lineage = target_lineage.split("; ")
-    candidate_lineage = candidate_lineage.split("; ")
-    tlen = len(target_lineage)
-    clen = len(candidate_lineage)
-    k = 0  # For tracking the overlapping positions
-    i = str_list_index(target_lineage, candidate_lineage[0])  # Lineage offset for the target
-    j = str_list_index(candidate_lineage, target_lineage[0])  # Lineage offset for the candidate
-
-    if i == tlen and j == clen:
-        if set(target_lineage).intersection(set(candidate_lineage)):
-            logging.debug("Target '%s' and candidate '%s' lineages were not overlapped " %
-                          (target_lineage, candidate_lineage))
-        return False
-
-    # There is a common string
-    while i+k < tlen and j+k < clen and target_lineage[i+k] == candidate_lineage[j+k]:
-        k += 1
-
-    # The verdict...
-    if i+k == len(target_lineage):
-        return True
-    else:
-        return False
-
-
-def select_rep_seqs(deduplicated_assignments: dict, test_sequences: dict, target_lineage=None):
+def select_rep_seqs(deduplicated_assignments: dict, test_sequences: dict,
+                    taxon_hierarchy: taxonomic_hierarchy.TaxonomicHierarchy, target_lineage=None) -> dict:
     """
     Function for creating a fasta-formatted dict from the accessions representing unique taxa in the test sequences
 
     :param deduplicated_assignments: dict of lineages mapped to a list of accessions
-    :param test_sequences: list of ReferenceSequence objects
+    :param test_sequences: Dictionary of ReferenceSequence objects indexed by their numerical identifiers
+    :param taxon_hierarchy: A TaxonomicHierarchy instance
     :param target_lineage: A taxonomic lineage to filter the lineages (and sequences) in deduplicated assignments
     :return: Dictionary containing accessions as keys and sequences as values
     """
     if target_lineage:
+        descendents = set()
         filtered_assignments = dict()
+        # Gather the set of all descendent lineages
+        target_taxon = target_lineage.split(taxon_hierarchy.lin_sep)[-1]
+        for line in [taxon.lineage() for taxon in taxon_hierarchy.get_taxon_descendents(target_taxon)]:  # type: list
+            descendents.add(taxon_hierarchy.lin_sep.join([taxon.prefix_taxon() for taxon in line]))
+        # Query the candidate lineages for whether they're in the target lineage's list of descendents
         for candidate_lineage in sorted(deduplicated_assignments):
-            if same_lineage(target_lineage, candidate_lineage):
+            if candidate_lineage in descendents:
                 filtered_assignments[candidate_lineage] = deduplicated_assignments[candidate_lineage]
     else:
         filtered_assignments = deduplicated_assignments
 
+    test_seq_lookup = {ref_seq.accession: ref_seq.sequence for ref_seq in test_sequences.values()}
+
     deduplicated_fasta_dict = dict()
     for lineage in sorted(filtered_assignments):
         for accession in filtered_assignments[lineage]:
-            matched = False
-            for _, ref_seq in test_sequences.items():  # type: (str, EntrezRecord)
-                if ref_seq.accession == accession:
-                    deduplicated_fasta_dict[accession] = ref_seq.sequence
-                    matched = True
-            if not matched:
+            try:
+                deduplicated_fasta_dict[accession] = test_seq_lookup[accession]
+            except KeyError:
                 logging.error("Unable to find accession (" + accession + ") in accession-lineage map\n")
                 sys.exit(21)
     return deduplicated_fasta_dict
@@ -349,43 +325,136 @@ def map_seqs_to_lineages(accession_lineage_map, deduplicated_fasta_dict):
     return seq_taxa_map
 
 
-def filter_queries_by_taxonomy(taxonomic_lineages):
-    """
-    Removes queries with duplicate taxa - to prevent the taxonomic composition of the input
-    from biasing the accuracy to over- or under-perform by classifying many sequences from very few groups.
-    Also removes taxonomies with "*nclassified" in their lineage
-
-    :param taxonomic_lineages: A list of lineages that need to be filtered
-    :return: A list that contains no more than 3 of each query taxonomy (arbitrarily normalized) and counts
-    """
-    unclassifieds = 0
-    classified = 0
-    unique_query_taxonomies = 0
-    lineage_enumerator = dict()
-    normalized_lineages = list()
-    for query_taxonomy in sorted(taxonomic_lineages):
-        can_classify = True
-        if re.search("unclassified", query_taxonomy, re.IGNORECASE):
-            # Remove taxonomic lineages that are unclassified at the Phylum level or higher
-            unclassified_depth = get_unclassified_rank(0, query_taxonomy.split("; "))
-            if unclassified_depth <= 3:
-                can_classify = False
-
-        if can_classify:
-            if query_taxonomy not in lineage_enumerator:
-                lineage_enumerator[query_taxonomy] = 0
-                classified += 1
-            if lineage_enumerator[query_taxonomy] < 3:
-                normalized_lineages.append(query_taxonomy)
-            lineage_enumerator[query_taxonomy] += 1
-        else:
-            unclassifieds += 1
-    unique_query_taxonomies += len(lineage_enumerator)
-
-    return normalized_lineages, unclassifieds, classified, unique_query_taxonomies
+# def filter_queries_by_taxonomy(taxonomic_lineages):
+#     """
+#     Removes queries with duplicate taxa - to prevent the taxonomic composition of the input
+#     from biasing the accuracy to over- or under-perform by classifying many sequences from very few groups.
+#     Also removes taxonomies with "*nclassified" in their lineage
+#
+#     :param taxonomic_lineages: A list of lineages that need to be filtered
+#     :return: A list that contains no more than 3 of each query taxonomy (arbitrarily normalized) and counts
+#     """
+#     unclassifieds = 0
+#     classified = 0
+#     unique_query_taxonomies = 0
+#     lineage_enumerator = dict()
+#     normalized_lineages = list()
+#     for query_taxonomy in sorted(taxonomic_lineages):
+#         can_classify = True
+#         if re.search("unclassified", query_taxonomy, re.IGNORECASE):
+#             # Remove taxonomic lineages that are unclassified at the Phylum level or higher
+#             unclassified_depth = get_unclassified_rank(0, query_taxonomy.split("; "))
+#             if unclassified_depth <= 3:
+#                 can_classify = False
+#
+#         if can_classify:
+#             if query_taxonomy not in lineage_enumerator:
+#                 lineage_enumerator[query_taxonomy] = 0
+#                 classified += 1
+#             if lineage_enumerator[query_taxonomy] < 3:
+#                 normalized_lineages.append(query_taxonomy)
+#             lineage_enumerator[query_taxonomy] += 1
+#         else:
+#             unclassifieds += 1
+#     unique_query_taxonomies += len(lineage_enumerator)
+#
+#     return normalized_lineages, unclassifieds, classified, unique_query_taxonomies
 
 
-def prep_graftm_ref_files(refpkg: ReferencePackage, tmp_dir: str, target_clade: str, executables: dict) -> dict:
+def run_clade_exclusion_treesapp(tt_obj: classy.TaxonTest, taxon_rep_seqs, ref_pkg, num_threads, executables,
+                                 trim_align=False, fresh=False, molecule_type="prot", min_seq_length=30) -> None:
+    if not os.path.isfile(tt_obj.classification_table):
+        # Copy reference files, then exclude all clades belonging to the taxon being tested
+        ce_refpkg = ref_pkg.clone(tt_obj.refpkg_path)
+        ce_refpkg.exclude_clade_from_ref_files(tt_obj.intermediates_dir, tt_obj.lineage,
+                                               executables, fresh)
+        # Write the query sequences
+        fasta.write_new_fasta(taxon_rep_seqs, tt_obj.test_query_fasta)
+        assign_args = ["-i", tt_obj.test_query_fasta, "-o", tt_obj.classifications_root,
+                       "--refpkg_dir", os.path.dirname(ce_refpkg.f__json),
+                       "-m", molecule_type, "-n", str(num_threads),
+                       "--min_seq_length", str(min_seq_length),
+                       "--overwrite", "--delete"]
+        if trim_align:
+            assign_args.append("--trim_align")
+
+        # TODO: Pause logging just to console and continue writing to log file
+        # Option 1. No logging to console or file
+        cl_log = logging.getLogger()
+        cl_log.disabled = True
+        try:
+            assign.assign(assign_args)
+        except:  # Just in case treesapp assign fails, just continue
+            pass
+        cl_log.disabled = False
+        if not os.path.isfile(tt_obj.classification_table):
+            # The TaxonTest object is maintained for record-keeping (to track # queries & classifieds)
+            logging.warning("TreeSAPP did not generate output for '{}'. Skipping.\n".format(tt_obj.lineage))
+            shutil.rmtree(tt_obj.classifications_root)
+            return
+    else:
+        # Valid number of queries and these sequences have already been classified
+        ce_refpkg = refpkg.ReferencePackage()
+        ce_refpkg.f__json = os.path.join(tt_obj.intermediates_dir,
+                                         '_'.join([ref_pkg.prefix, ref_pkg.refpkg_code,
+                                                   ref_pkg.date]),
+                                         ref_pkg.prefix + ref_pkg.refpkg_suffix)
+        ce_refpkg.slurp()
+
+    tt_obj.taxonomic_tree = ce_refpkg.all_possible_assignments()
+    if os.path.isfile(tt_obj.classification_table):
+        assigned_lines = file_parsers.read_classification_table(tt_obj.classification_table)
+        tt_obj.assignments = file_parsers.parse_assignments(assigned_lines)
+        tt_obj.filter_assignments(ref_pkg.prefix)
+        tt_obj.distances = parse_distances(assigned_lines)
+    else:
+        logging.error("marker_contig_map.tsv is missing from output directory '" +
+                      os.path.dirname(tt_obj.classification_table) + "'\n" +
+                      "Please remove this directory and re-run.\n")
+        sys.exit(21)
+    return
+
+
+def run_clade_exclusion_graftm(tt_obj: classy.TaxonTest, taxon_rep_seqs, ref_pkg: refpkg.ReferencePackage,
+                               graftm_classifier: str, num_threads: int, executables: dict) -> None:
+    # GraftM refpkg output files:
+    gpkg_refpkg_path = os.path.join(tt_obj.refpkg_path,
+                                    ref_pkg.prefix +
+                                    '_' + re.sub(' ', '_', tt_obj.taxon) + ".gpkg.refpkg") + os.sep
+    gpkg_tax_ids_file = gpkg_refpkg_path + ref_pkg.prefix + "_taxonomy.csv"
+    if not os.path.isfile(tt_obj.classification_table):
+        # Copy reference files, then exclude all clades belonging to the taxon being tested
+        output_paths = prep_graftm_ref_files(tmp_dir=tt_obj.intermediates_dir,
+                                             target_clade=tt_obj.lineage,
+                                             ref_pkg=ref_pkg,
+                                             executables=executables)
+
+        if not os.path.isdir(tt_obj.refpkg_path):
+            build_graftm_package(gpkg_path=tt_obj.refpkg_path,
+                                 tax_file=output_paths["filtered_tax_ids"],
+                                 mfa_file=output_paths["filtered_mfa"],
+                                 fa_file=output_paths["filtered_fasta"],
+                                 threads=num_threads)
+        # Write the query sequences
+        fasta.write_new_fasta(taxon_rep_seqs, tt_obj.test_query_fasta)
+
+        graftm_classify(tt_obj.test_query_fasta, tt_obj.refpkg_path, tt_obj.classifications_root,
+                        num_threads, graftm_classifier)
+
+        if not os.path.isfile(tt_obj.classification_table):
+            # The TaxonTest object is maintained for record-keeping (to track # queries & classifieds)
+            logging.warning("GraftM did not generate output for " + tt_obj.lineage + ". Skipping.\n")
+            shutil.rmtree(tt_obj.intermediates_dir)
+
+    tt_obj.taxonomic_tree = file_parsers.grab_graftm_taxa(gpkg_tax_ids_file)
+    graftm_assignments = file_parsers.read_graftm_classifications(tt_obj.classification_table)
+    tt_obj.assignments = {ref_pkg.prefix: graftm_assignments}
+    tt_obj.filter_assignments(ref_pkg.prefix)
+
+    return
+
+
+def prep_graftm_ref_files(ref_pkg: refpkg.ReferencePackage, tmp_dir: str, target_clade: str, executables: dict) -> dict:
     """
     From the original TreeSAPP reference package files, the necessary GraftM create input files are generated
     with all reference sequences related to the target_taxon removed from the multiple sequence alignment,
@@ -393,17 +462,17 @@ def prep_graftm_ref_files(refpkg: ReferencePackage, tmp_dir: str, target_clade:
 
     :param tmp_dir:  Path to write the intermediate files with target references removed
     :param target_clade: Taxonomic lineage of the clade that is being excluded from the reference package.
-    :param refpkg: A ReferencePackage instance for the reference package being tested
+    :param ref_pkg: A ReferencePackage instance for the reference package being tested
     :param executables: Dictionary of paths to dependency executables indexed by their names. Must include:
          'hmmbuild', 'FastTree' and 'raxml-ng'.
     :return: A dictionary providing paths to output files
     """
     # GraftM refpkg input paths:
-    output_paths = {"filtered_tax_ids": os.path.join(tmp_dir, refpkg.prefix + "_lineage_ids.txt"),
-                    "filtered_mfa": os.path.join(tmp_dir, refpkg.prefix + ".mfa"),
-                    "filtered_fasta": os.path.join(tmp_dir, refpkg.prefix + ".fa")}
+    output_paths = {"filtered_tax_ids": os.path.join(tmp_dir, ref_pkg.prefix + "_lineage_ids.txt"),
+                    "filtered_mfa": os.path.join(tmp_dir, ref_pkg.prefix + ".mfa"),
+                    "filtered_fasta": os.path.join(tmp_dir, ref_pkg.prefix + ".fa")}
 
-    ce_refpkg = refpkg.clone(clone_path=tmp_dir + refpkg.prefix + refpkg.refpkg_suffix)
+    ce_refpkg = ref_pkg.clone(clone_path=tmp_dir + ref_pkg.prefix + ref_pkg.refpkg_suffix)
 
     ce_refpkg.exclude_clade_from_ref_files(tmp_dir=tmp_dir, executables=executables, target_clade=target_clade)
 
@@ -416,10 +485,10 @@ def prep_graftm_ref_files(refpkg: ReferencePackage, tmp_dir: str, target_clade:
         taxa_handler.write("\n".join(lineage_info) + "\n")
 
     # Create and write the unaligned fasta file
-    ce_fasta = ce_refpkg.get_fasta()  # type: FASTA
-    write_new_fasta(fasta_dict=ce_fasta.fasta_dict, fasta_name=output_paths["filtered_mfa"])
+    ce_fasta = ce_refpkg.get_fasta()  # type: fasta.FASTA
+    fasta.write_new_fasta(fasta_dict=ce_fasta.fasta_dict, fasta_name=output_paths["filtered_mfa"])
     ce_fasta.unalign()
-    write_new_fasta(fasta_dict=ce_fasta.fasta_dict, fasta_name=output_paths["filtered_fasta"])
+    fasta.write_new_fasta(fasta_dict=ce_fasta.fasta_dict, fasta_name=output_paths["filtered_fasta"])
     return output_paths
 
 
