@@ -15,8 +15,8 @@ from numpy import var
 from treesapp import phylo_seq
 from treesapp.refpkg import ReferencePackage
 from treesapp import fasta
-from treesapp.utilities import median, write_dict_to_table, validate_new_dir, fetch_executable_path
-from treesapp.lca_calculations import determine_offset, optimal_taxonomic_assignment
+from treesapp import utilities as ts_utils
+from treesapp import lca_calculations
 from treesapp import entrez_utils
 from treesapp.wrapper import estimate_ml_model
 
@@ -77,7 +77,6 @@ def get_header_info(header_registry: dict, code_name=''):
     logging.info("Extracting information from headers... ")
     ref_records = dict()
     header_regexes = fasta.load_fasta_header_regexes(code_name)
-    # TODO: Fix parsing of combined EggNOG and custom headers such that the taxid is parsed from the "accession"
     for treesapp_id in sorted(header_registry.keys(), key=int):  # type: str
         original_header = header_registry[treesapp_id].original
         header_format_re, header_db, header_molecule = fasta.get_header_format(original_header, header_regexes)
@@ -358,7 +357,7 @@ class TreeSAPP:
         return info_string
 
     def set_output_dirs(self) -> None:
-        self.output_dir = validate_new_dir(self.output_dir)
+        self.output_dir = ts_utils.validate_new_dir(self.output_dir)
         self.final_output_dir = self.output_dir + "final_outputs" + os.sep
         self.var_output_dir = self.output_dir + "intermediates" + os.sep
         return
@@ -607,7 +606,7 @@ class TreeSAPP:
             dependencies += ["cmalign", "cmsearch", "cmbuild"]
 
         for dep in dependencies:
-            exec_paths[dep] = fetch_executable_path(dep, self.treesapp_dir)
+            exec_paths[dep] = ts_utils.fetch_executable_path(dep, self.treesapp_dir)
 
         return exec_paths
 
@@ -672,7 +671,7 @@ class TreeSAPP:
                     self.seq_lineage_map[accession[1:]] = self.seq_lineage_map.pop(accession)
 
             # Write the accession-lineage mapping file - essential for training too
-            write_dict_to_table(self.seq_lineage_map, self.acc_to_lin)
+            ts_utils.write_dict_to_table(self.seq_lineage_map, self.acc_to_lin)
             self.increment_stage_dir()
         elif self.stage_status("lineages") is False and os.path.isfile(self.acc_to_lin):
             logging.info("Reading cached lineages in '{}'... ".format(self.acc_to_lin))
@@ -695,6 +694,7 @@ class Updater(TreeSAPP):
         self.cluster_input = ""  # Used only if resolve is True
         self.clusters_prefix = ""  # Used only if resolve is True
         self.updated_refpkg_path = ""
+        self.training_dir = ""
         # self.rank_depth_map = None
         self.prop_sim = 1.0
         self.min_length = 0  # The minimum sequence length for a classified sequence to be included in the refpkg
@@ -702,7 +702,8 @@ class Updater(TreeSAPP):
 
         # Stage names only holds the required stages; auxiliary stages (e.g. RPKM, update) are added elsewhere
         self.stages = {0: ModuleFunction("lineages", 0),
-                       1: ModuleFunction("rebuild", 1)}
+                       1: ModuleFunction("rebuild", 1),
+                       2: ModuleFunction("train", 2)}
 
     def decide_stage(self, args):
         """
@@ -711,10 +712,13 @@ class Updater(TreeSAPP):
         This function ensures all the required inputs are present for beginning at the desired first stage,
         otherwise, the pipeline begins at the first possible stage to continue and ends once the desired stage is done.
 
-        If a
         :return: None
         """
         self.acc_to_lin = self.var_output_dir + os.sep + "accession_id_lineage_map.tsv"
+
+        if not self.input_sequences:
+            self.change_stage_status("train", False)
+
         self.validate_continue(args)
         return
 
@@ -772,15 +776,16 @@ class Creator(TreeSAPP):
         self.training_dir = ""
 
         # Stage names only holds the required stages; auxiliary stages (e.g. RPKM, update) are added elsewhere
-        self.stages = {0: ModuleFunction("search", 0),
-                       1: ModuleFunction("lineages", 1),
-                       2: ModuleFunction("clean", 2),
-                       3: ModuleFunction("cluster", 3),
-                       4: ModuleFunction("build", 4),
-                       5: ModuleFunction("evaluate", 5),
-                       6: ModuleFunction("support", 6),
-                       7: ModuleFunction("train", 7),
-                       8: ModuleFunction("update", 8)}
+        self.stages = {0: ModuleFunction("deduplicate", 0),
+                       1: ModuleFunction("search", 1),
+                       2: ModuleFunction("lineages", 2),
+                       3: ModuleFunction("clean", 3),
+                       4: ModuleFunction("cluster", 4),
+                       5: ModuleFunction("build", 5),
+                       6: ModuleFunction("evaluate", 6),
+                       7: ModuleFunction("support", 7),
+                       8: ModuleFunction("train", 8),
+                       9: ModuleFunction("update", 9)}
 
     def decide_stage(self, args):
         """
@@ -789,9 +794,11 @@ class Creator(TreeSAPP):
         This function ensures all the required inputs are present for beginning at the desired first stage,
         otherwise, the pipeline begins at the first possible stage to continue and ends once the desired stage is done.
 
-        If a
         :return: None
         """
+        if not args.dedup:
+            self.change_stage_status("deduplicate", False)
+
         if not args.profile:
             self.change_stage_status("search", False)
         if args.pc:
@@ -1030,7 +1037,7 @@ class TaxonTest:
     def get_optimal_assignment(self):
         if self.lineage.split('; ')[0] != "r__Root":
             self.lineage = "; ".join(["r__Root"] + self.lineage.split("; "))
-        return optimal_taxonomic_assignment(self.taxonomic_tree, self.lineage)
+        return lca_calculations.optimal_taxonomic_assignment(self.taxonomic_tree, self.lineage)
 
     def summarise_taxon_test(self):
         summary_string = "Test for taxonomic lineage '" + self.lineage + "':\n" + \
@@ -1235,19 +1242,19 @@ class Evaluator(TreeSAPP):
             distance_summary = ["Rank\tType\tMean\tMedian\tVariance",
                                 "\t".join([rank, "Distal",
                                            str(round(sum(distals) / float(n_dists), 4)),
-                                           str(round(median(distals), 4)),
+                                           str(round(ts_utils.median(distals), 4)),
                                            str(round(float(var(distals)), 4))]),
                                 "\t".join([rank, "Pendant",
                                            str(round(sum(pendants) / float(n_dists), 4)),
-                                           str(round(median(pendants), 4)),
+                                           str(round(ts_utils.median(pendants), 4)),
                                            str(round(float(var(pendants)), 4))]),
                                 "\t".join([rank, "Tip",
                                            str(round(sum(tips) / float(n_dists), 4)),
-                                           str(round(median(tips), 4)),
+                                           str(round(ts_utils.median(tips), 4)),
                                            str(round(float(var(tips)), 4))]),
                                 "\t".join([rank, "Total",
                                            str(round(sum(totals) / float(n_dists), 4)),
-                                           str(round(median(totals), 4)),
+                                           str(round(ts_utils.median(totals), 4)),
                                            str(round(float(var(totals)), 4))])]
             sys.stdout.write("\n".join(distance_summary) + "\n")
             return distals, pendants, tips
@@ -1353,7 +1360,7 @@ class Evaluator(TreeSAPP):
                         if optimal == classified:
                             offset = 0
                         else:
-                            offset = determine_offset(classified, optimal)
+                            offset = lca_calculations.determine_offset(classified, optimal)
                         if offset > 7:
                             # This shouldn't be possible since there are no more than 7 taxonomic ranks
                             logging.error("Offset found to be greater than what is possible (" + str(offset) + ").\n" +
@@ -1464,11 +1471,12 @@ class Abundance(TreeSAPP):
     def __init__(self):
         super(Abundance, self).__init__("abundance")
         self.target_refpkgs = list()
-        self.classified_nuc_seqs = ""
+        self.ref_nuc_seqs = ""
         self.classifications = ""
         self.aln_file = ""
         self.append_abundance = False
         self.fq_suffix_re = re.compile(r"([._-])+(pe|fq|fastq|fwd|R1|1)$")
+        self.idx_extensions = ["amb", "ann", "bwt", "pac", "sa"]
         self.stages = {0: ModuleFunction("align_map", 0),
                        1: ModuleFunction("sam_sum", 1),
                        2: ModuleFunction("summarise", 2)}
@@ -1481,19 +1489,28 @@ class Abundance(TreeSAPP):
         if len(glob(self.final_output_dir + "*_classified.fna")) < 1:
             logging.error("Unable to find classified ORF nucleotide sequences in '{}'.\n".format(self.final_output_dir))
             sys.exit(5)
-        self.classified_nuc_seqs = glob(self.final_output_dir + "*_classified.fna")[0]
+        self.ref_nuc_seqs = ts_utils.match_file(os.path.join(self.var_output_dir, "orf-call", "*_ORFs.fna"))
 
         self.classifications = self.final_output_dir + self.classification_tbl_name
 
         if not os.path.isdir(self.var_output_dir):
             os.makedirs(self.var_output_dir)
 
+        # Set the directory paths for each stage. Usually done in TreeSAPP.check_previous_output() but can't use here.
+        for stage_order, stage in self.stages.items():  # type: (int, ModuleFunction)
+            stage.dir_path = self.var_output_dir + stage.name + os.sep
+
+        # Remove the directory containing SAM and BWA index files
+        if os.path.isdir(self.stage_lookup("align_map").dir_path) and args.overwrite:
+            logging.debug("Removing directory with BWA outputs '{}'.\n".format(self.stage_lookup("align_map").dir_path))
+            rmtree(self.stage_lookup("align_map").dir_path)
+
         self.aln_file = self.stage_lookup("align_map").dir_path + \
-                        '.'.join(os.path.basename(self.classified_nuc_seqs).split('.')[0:-1]) + ".sam"
+                        '.'.join(os.path.basename(self.ref_nuc_seqs).split('.')[0:-1]) + ".sam"
 
         if len(args.reverse) > 0:
             if len(args.reads) != len(args.reverse):
-                logging.error("Number of fastq files differs between reads () and reverse () arguments!.\n"
+                logging.error("Number of fastq files differs between reads ({}) and reverse ({}) arguments!.\n"
                               "".format(len(args.reads), len(args.reverse)))
                 sys.exit(3)
 
@@ -1518,16 +1535,24 @@ class Abundance(TreeSAPP):
         if not clean_up:
             return
 
-        files_to_be_deleted = [self.aln_file]
+        files_to_be_deleted = glob(self.stage_lookup("align_map").dir_path + "*.sam")
         files_to_be_deleted += glob(self.stage_lookup("align_map").dir_path + "*.stderr")
+        for ext in self.idx_extensions:
+            files_to_be_deleted += glob(self.stage_lookup("align_map").dir_path + "*." + ext)
 
-        for file_path in files_to_be_deleted:
+        if self.aln_file not in files_to_be_deleted:
+            files_to_be_deleted.append(self.aln_file)
+
+        for file_path in set(files_to_be_deleted):
             if os.path.exists(file_path):
                 os.remove(file_path)
         return
 
     def strip_file_to_sample_name(self, file_path) -> None:
-        file_prefix = '.'.join(os.path.basename(file_path).split('.')[:-1])
+        file_name, suffix = os.path.splitext(os.path.basename(file_path))
+        if suffix == ".gz":
+            file_name, suffix = os.path.splitext(file_name)
+        file_prefix = '.'.join(file_name.split('.'))
         self.sample_prefix = self.fq_suffix_re.sub('', file_prefix)
         return
 
