@@ -627,11 +627,7 @@ def fetch_lineages_from_taxids(entrez_records: list, t_hierarchy=None) -> None:
             lineage_ex += [{"ScientificName": tax_organism, "Rank": tax_rank}]
 
         taxon = t_hierarchy.feed(tax_lineage, lineage_ex)  # type: Taxon
-        # Ensure all of the ranks in the lineage have been incremented,
-        # whether they're part of the NCBI taxonomy or not. Necessary for 'r__Root'.
-        for t in taxon.lineage():  # type: Taxon
-            if t.parent and t.coverage > t.parent.coverage:
-                t.parent.coverage += 1
+        taxon.increment_absent_ancestral_coverage()
         lineage_anno = t_hierarchy.emit(taxon.prefix_taxon(), True)
 
         try:
@@ -1255,7 +1251,7 @@ def map_orf_lineages(seq_lineage_tbl: str, header_registry: dict, refpkg_name=No
     :param refpkg_name: The reference package's name
     :return: A dictionary mapping each classified sequence to a lineage and list of TreeSAPP IDs that were mapped
     """
-    logging.info("Mapping assigned sequences to provided taxonomic lineages:\n")
+    logging.info("Mapping sequence names to provided taxonomic lineages in {}:\n".format(seq_lineage_tbl))
     seq_lineage_map = read_seq_taxa_table(seq_lineage_tbl)
     classified_seq_lineage_map = dict()
     treesapp_nums = list(header_registry.keys())
