@@ -473,8 +473,8 @@ def assignments_to_pqueries(classified_lines: list) -> dict:
         pquery = PQuery()
         con_place = PhyloPlace()
         try:
-            _, pquery.seq_name, pquery.ref_name, pquery.start, pquery.end, pquery.recommended_lineage, \
-            _, con_place.edge_num, pquery.evalue, con_place.like_weight_ratio, pquery.avg_evo_dist, pquery.distances = fields
+            _, pquery.seq_name, pquery.ref_name, pquery.start, pquery.end, pquery.recommended_lineage, pquery.abundance,\
+            con_place.edge_num, pquery.evalue, con_place.like_weight_ratio, pquery.avg_evo_dist, pquery.distances = fields
         except ValueError:
             logging.error("Bad line in classification table:\n" +
                           '\t'.join(fields) + "\n")
@@ -482,6 +482,7 @@ def assignments_to_pqueries(classified_lines: list) -> dict:
         pquery.place_name = "{}|{}|{}_{}".format(pquery.seq_name, pquery.ref_name, pquery.start, pquery.end)
         pquery.end = int(pquery.end)
         pquery.start = int(pquery.start)
+        pquery.abundance = float(pquery.abundance)
         pquery.seq_len = pquery.end - pquery.start
         pquery.lct = pquery.recommended_lineage
         con_place.distal_length, con_place.pendant_length, con_place.mean_tip_length = [float(d) for d in
@@ -523,15 +524,15 @@ def abundify_tree_saps(tree_saps: dict, abundance_dict: dict):
     abundance_mapped_acc = 0
     for placed_seqs in tree_saps.values():  # type: list
         for pquery in placed_seqs:  # type: PQuery
-            if not pquery.abundance:
-                # Filter out RPKMs for contigs not associated with the target marker
+            try:
+                pquery.abundance = abundance_dict[pquery.place_name]
+                abundance_mapped_acc += 1
+            except KeyError:
                 try:
-                    pquery.abundance = abundance_dict[pquery.place_name]
+                    pquery.abundance = abundance_dict[pquery.seq_name]
                     abundance_mapped_acc += 1
                 except KeyError:
                     pquery.abundance = 0.0
-            else:
-                abundance_mapped_acc += 1
 
     if abundance_mapped_acc == 0:
         logging.warning("No placed sequences with abundances identified.\n")
